# Redmine - project management software
# Copyright (C) 2006-2013  Jean-Philippe Lang
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

class News < ActiveRecord::Base
  include Redmine::SafeAttributes
  belongs_to :project,:touch => true
  include ApplicationHelper
  has_many_kindeditor_assets :assets, :dependent => :destroy
  #added by nwb
  belongs_to :course,:touch => true
  belongs_to :author, :class_name => 'User', :foreign_key => 'author_id'
  has_many :comments, :as => :commented, :dependent => :destroy, :order => "created_on"
  # fq
  has_many :acts, :class_name => 'Activity', :as => :act, :dependent => :destroy
  # 被ForgeActivity虚拟关联
  has_many :forge_acts, :class_name => 'ForgeActivity',:as =>:forge_act ,:dependent => :destroy
  # 课程动态
  has_many :course_acts, :class_name => 'CourseActivity',:as =>:course_act ,:dependent => :destroy
  # end
  # 课程/项目消息关联
  has_many :course_messages, :class_name =>'CourseMessage', :as => :course_message, :dependent => :destroy
  has_many :forge_messages, :class_name => 'ForgeMessage', :as => :forge_message, :dependent => :destroy
  #end

  has_many :ActivityNotifies,:as => :activity, :dependent => :destroy

  validates_presence_of :title, :description
  validates_length_of :title, :maximum => 60
  validates_length_of :summary, :maximum => 255
  validates_length_of :description, :maximum => 10000

  acts_as_attachable :delete_permission => :manage_news
  acts_as_searchable :columns => ['title', 'summary', "#{table_name}.description"], :include => :project
  acts_as_event :url => Proc.new {|o| {:controller => 'news', :action => 'show', :id => o.id}}
  acts_as_activity_provider :find_options => {:include => [:project, :author]},
                            :author_key => :author_id
  #added by nwb
  #课程新闻独立于项目
  acts_as_activity_provider :type => 'course_news',
                            :find_options => {:include => [:course, :author]},
                            :author_key => :author_id
  acts_as_watchable

  after_create :act_as_activity,:act_as_forge_activity, :act_as_course_activity,:act_as_system_message, :add_author_as_watcher, :send_mail

  after_destroy :delete_kindeditor_assets

  scope :visible, lambda {|*args|
    includes(:project).where(Project.allowed_to_condition(args.shift || User.current, :view_news, *args))
  }

  scope :course_visible, lambda {|*args|
    includes(:course).where(Course.allowed_to_condition(args.shift || User.current, :view_course_news, *args))
  }
  safe_attributes 'title', 'summary', 'description', 'sticky'

  def visible?(user=User.current)
    !user.nil? && user.allowed_to?(:view_news, project)
  end

  # Returns true if the news can be commented by user
  def commentable?(user=User.current)
    user.allowed_to?(:comment_news, project)
  end

  def recipients
    project.users.select {|user| user.notify_about?(self)}.map(&:mail)
  end

  # returns latest news for projects visible by user
  def self.latest(user = User.current, count = 5)
    visible(user).includes([:author, :project]).order("#{News.table_name}.created_on DESC").limit(count).all
  end

  # 新闻的短描述信息
  def short_description(length = 255)
    description.gsub(/<\/?.*?>/,"").html_safe if description
    #description.gsub(/^(.{#{length}}[^\n\r]*).*$/m, '\1...').strip if description
    #description
  end

  def set_notify_id(notify_id)
    @notify_id= notify_id
  end
  def get_notify_id()
    return @notify_id
  end
  def set_notify_is_read(notify_is_read)
    @notify_is_read = notify_is_read
  end
  def get_notify_is_read()
    return @notify_is_read
  end

  private

  def add_author_as_watcher
    Watcher.create(:watchable => self, :user => author)
  end
  ## fq
  def act_as_activity
    self.acts << Activity.new(:user_id => self.author_id)
  end

  # Time 2015-02-27 15:48:17
  # Author lizanle
  # Description 公用表中也要记录
  def act_as_forge_activity
    # 如果是project为空，那么是课程相关的，不需要保存
    if self.project
      self.forge_acts << ForgeActivity.new(:user_id => self.author_id,
                                             :project_id => self.project.id)
    end
  end

  #课程动态公共表记录
  def act_as_course_activity
    if self.course
      self.course_acts << CourseActivity.new(:user_id => self.author_id,:course_id => self.course_id)
    end
  end

  #课程/项目通知 消息发送
  #消息发送原则：除了消息的发布者，课程的其它成员都能收到消息提醒
  def act_as_system_message
    if self.course
      self.course.members.each do |m|
        if m.user_id != self.author_id
          self.course_messages << CourseMessage.new(:user_id => m.user_id, :course_id => self.course_id, :viewed => false)
        end
      end
    else
      if !self.project.nil?
        self.project.members.each do |m|
          if m.user_id != self.author_id
            self.forge_messages << ForgeMessage.new(:user_id => m.user_id, :project_id => self.project_id, :viewed => false)
          end
        end
      end
    end
  end

  # Time 2015-03-31 13:50:54
  # Author lizanle
  # Description 删除news后删除对应的资源
  def delete_kindeditor_assets
    delete_kindeditor_assets_from_disk self.id,OwnerTypeHelper::NEWS
  end

  def send_mail
    Mailer.run.news_added(self) if Setting.notified_events.include?('news_added')
  end

end